#include "State.h"
#include <iostream>

State::State()
{
}

void StartRoomState::OnEntry(Game *context)
{
	if (m_first_time)
	{
		std::cout << std::endl << "welcome to state machine quest - a simple but functional adventure into the realm of finite state machines." << std::endl;
		std::cout << "you are in a small room with two doors, one on the east side, the other on the south side." << std::endl;
		m_first_time = false;
	}
	else
		std::cout << "you entered a small room with two doors, one on the east side, the other on the south side." << std::endl;
}

void StartRoomState::OnExit(Game *context)
{
	std::cout << "you exited the small room with two doors" << std::endl;
}

void StartRoomState::OnCommand(Game *context, std::string const &command)
{
	if (command == "go east")
	{
		std::cout << "going through the door on the east side..." << std::endl;
		
		context->SetState(Game::m_chest_room_state);
	}
	else if (command == "go west")
		std::cout << "ouch! you bumped into a brick wall." << std::endl;
	else if (command == "go north")
		std::cout << "ouch! you bumped into a brick wall." << std::endl;
	else if (command == "go south")
	{
		std::cout << "going through the door on the south side..." << std::endl;
		
		context->SetState(Game::m_key_room_state);
	}
	else if (command == "look around")
	{
		std::cout << "it's a small room with a door on the east side and another door on the south side" << std::endl;
		if (!m_apple_picked)
			std::cout << "there's an apple in the room" << std::endl;
	}			
	else if (command == "pick up apple")
	{
		if (!m_apple_picked)
		{
			std::cout << "you picked up a juicy red apple" << std::endl;
			context->SetApple(true);
			m_apple_picked = true;
		}
		else	
			std::cout << "you already picked up the apple." << std::endl;	
	}
	else if (command == "eat apple")
	{
		if (context->HasApple())                                 
		{
			std::cout << "mmm.. delicious!" << std::endl;
			context->SetApple(false);
		}
		else
			std::cout << "I can't eat an apple, too bad I haven't got one!" << std::endl;	
	}
	else if (command == "use key")
	{
		if (context->HasKey())                                 
			std::cout << "ther's nothing to use the key with." << std::endl;
		else
			std::cout << "Which key? I have got no key!" << std::endl;	
	}
	else
		std::cout << "sorry, I didn't understand. say again?" << std::endl;
}

void KeyRoomState::OnEntry(Game *context)
{
	std::cout << "you entered a dimly lit room. There's a door on the north side." << std::endl;
}

void KeyRoomState::OnExit(Game *context)
{
	std::cout << "you exited the dimly lit room." << std::endl;
}

void KeyRoomState::OnCommand(Game *context, std::string const &command)
{
	if (command == "go east")
		std::cout << "ouch! you bumped into a brick wall." << std::endl;
	else if (command == "go west")
		std::cout << "ouch! you bumped into a brick wall." << std::endl;
	else if (command == "go north")
	{
		std::cout << "going through the door on the north side..." << std::endl;
		
		context->SetState(Game::m_start_room_state);
	}
	else if (command == "go south")
		std::cout << "ouch! you bumped into a brick wall." << std::endl;
	else if (command == "look around")
	{
		std::cout << "it's a dimly lit room with a door on the north side." << std::endl;
		if (!m_key_picked)
			std::cout << "there's rusted key on the floor." << std::endl;
	}	
	else if (command == "pick up key")
	{
		if (!m_key_picked)
		{
			std::cout << "you picked up an old, rusted key." << std::endl;
			context->SetKey();
			m_key_picked = true;
		}
		else	
			std::cout << "you already picked up the key." << std::endl;		
	}
	else if (command == "eat apple")
	{
		if (context->HasApple())                                 
		{
			std::cout << "mmm.. delicious!" << std::endl;
			context->SetApple(false);
		}
		else
			std::cout << "I can't eat an apple, too bad I haven't got one!" << std::endl;	
	}
	else if (command == "use key")
	{
		if (context->HasKey())                                 
			std::cout << "ther's nothing to use the key with." << std::endl;
		else
			std::cout << "Which key? I have got no key!" << std::endl;	
	}
	else
		std::cout << "sorry, I didn't understand. say again?" << std::endl;
}

void ChestRoomState::OnEntry(Game *context)
{
		std::cout << "you entered a big room with tapestries on the walls. There's a door on the west side." << std::endl;	
}

void ChestRoomState::OnExit(Game *context)
{
		std::cout << "you exited the big room with the tapestries." << std::endl;
}

void ChestRoomState::OnCommand(Game *context, std::string const &command)
{
	if (command == "go east")
		std::cout << "ouch! you bumped into a brick wall." << std::endl;
	else if (command == "go west")
	{
		std::cout << "going through the door on the west side..." << std::endl;
		
		context->SetState(Game::m_start_room_state);
	}
	else if (command == "go north")
		std::cout << "ouch! you bumped into a brick wall." << std::endl;
	else if (command == "go south")
		std::cout << "ouch! you bumped into a brick wall." << std::endl;
	else if (command == "look around")
		std::cout << "there's a chest on the far side of the room." << std::endl;
	else if (command == "pick up key")
	{
			std::cout << "you picked up an old, rusted key." << std::endl;
			context->SetKey();
	}
	else if (command == "eat apple")
	{
		if (context->HasApple())                                 
		{
			std::cout << "mmm.. delicious!" << std::endl;
			context->SetApple(false);
		}
		else
			std::cout << "I can't eat an apple, too bad I haven't got one!" << std::endl;	
	}
	else if (command == "use key")
	{
		if (context->HasKey())     
		{			
			std::cout << "you opened the chest. There's a treasure in it!" << std::endl;
			std::cout << "congratulations! you win the great state machine quest!" << std::endl;
			context->Win();
		}
		else
			std::cout << "Which key? I have got no key!" << std::endl;	
	}
	else
		std::cout << "sorry, I didn't understand. say again?" << std::endl;
}

State *Game::m_start_room_state = new StartRoomState;         
State *Game::m_key_room_state = new KeyRoomState;   ;
State *Game::m_chest_room_state = new ChestRoomState;   

Game::Game() 
{	
	m_current_state = m_start_room_state;
	m_current_state->OnEntry(this);
}

void Game::OnCommand(std::string const &command)
{
	m_current_state->OnCommand(this, command);
}

void Game::SetState(State *new_state)
{
	// leave the current state - execute exit action
	m_current_state->OnExit(this);
	
	// state transistion
	m_current_state = new_state;
	
	// enter the new state - execute enter action
	m_current_state->OnEntry(this);
}