#include <string>

class Game;

class State
{
public:
	virtual ~State() {}
	virtual void OnCommand(Game *context, std::string const &command) = 0;
	virtual void OnEntry(Game *context) = 0;
	virtual void OnExit(Game *context) = 0;
protected:
	State();
};

class StartRoomState : public State
{
public:
	void OnCommand(Game *context, std::string const &command) override;
	void OnEntry(Game *context) override;
	void OnExit(Game *context) override;
private:
	bool m_first_time = true;
	bool m_apple_picked = false;
};

class KeyRoomState : public State
{
public:
	void OnCommand(Game *context, std::string const &command) override;
	void OnEntry(Game *context) override;
	void OnExit(Game *context) override;
private:
	bool m_key_picked = false;
};

class ChestRoomState : public State
{
public:
	void OnCommand(Game *context, std::string const &command) override;
	void OnEntry(Game *context) override;
	void OnExit(Game *context) override;
};

class Game
{
public:
	Game();
	~Game() {}
	void OnCommand(std::string const &command);
	void SetState(State *new_state);           // state setter function
	bool HasApple() { return m_has_apple; }    // getters/setters for extended state variables
	void SetApple(bool b) { m_has_apple = b; } 
	bool HasKey() { return m_has_key; }
	void SetKey() { m_has_key = true; } 
	void Win() { m_win = true; }
	bool HasWon() {return m_win; }
	static State *m_start_room_state;          // concrete state objects
	static State *m_key_room_state;
	static State *m_chest_room_state;
private:
	State *m_current_state;                    // current state (state variable)
	bool m_has_apple = false;                  // extended state variables
	bool m_has_key = false;
	bool m_win = false;
};



